package com.laguerre.kan;

import java.util.Arrays;

import android.opengl.Matrix;

public final class CommonRenderingPrms {
	private final int[] viewSize = new int[2];
	private final float[] viewMatrix = new float[16];
	private final float[] invViewMatrix = new float[16];
	private final float[] projMatrix = new float[16];
	private final float[] invProjMatrix = new float[16];
	private final float[] viewProjMatrix = new float[16];
	private final float[] invViewProjMatrix = new float[16];

	public CommonRenderingPrms() {
		Arrays.fill(this.viewSize, 100);
		Matrix.setIdentityM(this.viewMatrix, 0);
		Matrix.setIdentityM(this.invViewMatrix, 0);
		Matrix.setIdentityM(this.projMatrix, 0);
		Matrix.setIdentityM(this.invProjMatrix, 0);
		Matrix.setIdentityM(this.viewProjMatrix, 0);
		Matrix.setIdentityM(this.invViewProjMatrix, 0);
	}

	public void setViewSize(int width, int height) {
		this.viewSize[0] = width;
		this.viewSize[1] = height;
	}

	public void setViewSize(int[] viewSize) {
		System.arraycopy(viewSize, 0, this.viewSize, 0, 2);
	}

	public void setViewSizeWidth(int width) {
		this.viewSize[0] = width;
	}

	public void setViewSizeHeight(int height) {
		this.viewSize[1] = height;
	}

	public int[] getSharedViewSize() {
		return this.viewSize;
	}

	public void setViewMatrix(float[] viewMatrix) {
		setViewMatrix(viewMatrix, null);
	}

	public void setViewMatrix(float[] viewMatrix, float[] invViewMatrix) {
		setMatrixes(viewMatrix, this.viewMatrix, invViewMatrix, this.invViewMatrix);
	}

	public void setViewProjMatrix(float[] projViewMatrix) {
		setViewProjMatrix(projViewMatrix, null);
	}

	public void setViewProjMatrix(float[] viewProjMatrix, float[] invViewProjMatrix) {
		setMatrixes(viewProjMatrix, this.viewProjMatrix, invViewProjMatrix, this.invViewProjMatrix);
	}

	public float[] getSharedViewMatrix() {
		return this.viewMatrix;
	}

	public float[] getSharedInvViewMatrix() {
		return this.invViewMatrix;
	}

	public void setProjMatrix(float[] projMatrix) {
		setProjMatrix(projMatrix, null);
	}

	public void setProjMatrix(float[] projMatrix, float[] invProjMatrix) {
		setMatrixes(projMatrix, this.projMatrix, invProjMatrix, this.invProjMatrix);
	}

	public float[] getSharedProjMatrix() {
		return this.projMatrix;
	}

	public float[] getSharedInvProjMatrix() {
		return this.invProjMatrix;
	}

	public float[] getSharedViewProjMatrix() {
		return this.viewProjMatrix;
	}

	public float[] getSharedInvViewProjMatrix() {
		return this.invViewProjMatrix;
	}

	private void setMatrixes(float[] src, float[] fieldDst, float[] invSrc, float[] fieldInvDst) {
		System.arraycopy(src, 0, fieldDst, 0, src.length);
		if (invSrc != null) {
			System.arraycopy(invSrc, 0, fieldInvDst, 0, src.length);
		} else {
			Matrix.invertM(fieldInvDst, 0, src, 0);
		}
	}
}
