package com.laguerre.mytest;

import com.laguerre.kan.Behaviour;
import com.laguerre.kan.ImageEffects;
import com.laguerre.kan.PassShader;
import com.laguerre.kan.Shader;
import com.laguerre.kan.SurfaceInfo;
import com.laguerre.kan.Utils;

public class PerlinImageEffect extends Behaviour {
	private final Shader perlinShader = new Shader();
	private final int[] screenDim = {0, 0};
	private float myTime;
	private final float timeStep = 0.01f;

	@Override
	public void onAwake() {
		final PassShader passPerlin = this.perlinShader.addPassShader(0);
		passPerlin.createProgram();
		passPerlin.attachVertexShader(ImageEffects.getBasicVertexShaderCode());
		passPerlin.attachFragmentShader(Utils.readTextFileFromRawResource(
				Utils.getContext(), R.raw.perlin_noise_f_glsl));
		passPerlin.linkProgram();

		ImageEffects.getInstance().useAlternativeShader(this.perlinShader);

		this.myTime = 0.0f;
	}

	@Override
	public void onBeforeRenderingImageEffectsPass(int passNum) {
		this.myTime += this.timeStep;
		if (this.myTime > Float.MAX_VALUE - 10f) {
			this.myTime = 0.0f;
		}

		SurfaceInfo.getInstance().getSurfaceSize(this.screenDim);
		final float[] resolution = new float[] { this.screenDim[0], this.screenDim[1] };
		this.perlinShader.setShaderVector("resolution", resolution);
		this.perlinShader.setShaderValue("time", this.myTime);
	}

	@Override
	public void onDestroy() {
		perlinShader.destroyResources();
	}
}
