package com.laguerre.mytest;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.ShortBuffer;

import com.laguerre.kan.ManualMesh;

public class SimpleTriangle extends ManualMesh {
	private FloatBuffer vertexBuffer;
	private FloatBuffer normalBuffer;
	private FloatBuffer texCoordBuffer;
	private ShortBuffer drawListBuffer;

    private static final float[] triangleCoords = {
         0.0f,  0.622008459f, 0.0f,   // top
        -0.5f, -0.311004243f, 0.0f,   // bottom left
         0.5f, -0.311004243f, 0.0f    // bottom right
    };
    private static final float[] normalCoords = {
        0.0f, 0.0f, 1.0f, 1.0f,   // top
        0.0f, 0.0f, 1.0f, 1.0f,   // bottom left
        0.0f, 0.0f, 1.0f, 1.0f    // bottom right
   };
    private static final float[] texCoords = {
    	0.5f, 1.0f,
    	0.0f, 0.0f,
    	1.0f, 0.0f
    };
    private static short[] drawList = {
		0, 1, 2
    };

    private final int vertexStride = 4 * Float.SIZE / 8;
    private final int normalStride = 3 * Float.SIZE / 8;;

    @Override
    public int getNbOfSubMeshes() {
    	return 1;
    }

	@Override
	public void build() {
        ByteBuffer bb = ByteBuffer.allocateDirect(triangleCoords.length * Float.SIZE / 8);
        bb.order(ByteOrder.nativeOrder());
        vertexBuffer = bb.asFloatBuffer();
        vertexBuffer.put(triangleCoords);
        vertexBuffer.position(0);

        bb = ByteBuffer.allocateDirect(normalCoords.length * Float.SIZE / 8);
        bb.order(ByteOrder.nativeOrder());
        normalBuffer = bb.asFloatBuffer();
        normalBuffer.put(normalCoords);
        normalBuffer.position(0);

        bb = ByteBuffer.allocateDirect(texCoords.length * Float.SIZE / 8);
        bb.order(ByteOrder.nativeOrder());
        texCoordBuffer = bb.asFloatBuffer();
        texCoordBuffer.put(texCoords);
        texCoordBuffer.position(0);

        bb = ByteBuffer.allocateDirect(drawList.length * Short.SIZE / 8);
        bb.order(ByteOrder.nativeOrder());
        this.drawListBuffer = bb.asShortBuffer();
    	this.drawListBuffer.put(drawList);
    	this.drawListBuffer.position(0);
	}

	@Override
	public final int getVertexCount(int subMeshId) {
		return 3;
	}

	@Override
	public final int getVertexStride(int subMeshId) {
		return this.vertexStride;
	}

	@Override
	public final FloatBuffer getSharedVertexBuffer(int subMeshId) {
		return this.vertexBuffer;
	}

	@Override
	public int getNormalStride(int subMeshId) {
		return this.normalStride;
	}

	@Override
	public FloatBuffer getSharedNormalBuffer(int subMeshId) {
		return this.normalBuffer;
	}

	@Override
	public final FloatBuffer getSharedTexCoordBuffer(int subMeshId, int textureId) {
		return this.texCoordBuffer;
	}

	@Override
	public int getSharedTexCoordStride(int subMeshId, int textureId) {
		return 2 * 4;
	}

	@Override
	public int getDrawListCount(int subMeshId) {
		return 3;
	}

	@Override
	public ShortBuffer getSharedDrawListBuffer(int subMeshId) {
		return this.drawListBuffer;
	}
}
