package com.laguerre.mytest;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.ShortBuffer;

import com.laguerre.kan.PassShader;
import com.laguerre.kan.RenderTexture;
import com.laguerre.kan.Shader;
import com.laguerre.kan.Texture;
import com.laguerre.kan.Utils;
import com.laguerre.kan.Camera.FrameBufferIncompleteException;

import android.opengl.GLES20;

public final class TestFBO {
	private static TestFBO instance = null;
	public static TestFBO getInstance() {
		if (instance == null) {
			instance = new TestFBO();
		}
		return instance;
	}
	public static void reset() {
		instance = null;
	}


    private static final String vShaderBasicCode =
    		"attribute vec4 position;" +
    		"void main() {" +
    		"  gl_Position = position;" +
    		"  gl_Position.x *= 0.5f;" +
    		"}";

    private static final String fShaderBasicCode =
    		"precision mediump float;" +
    		"void main() {" +
    		"  gl_FragColor = vec4(0.5f, 0.2f, 0.8f, 1.0f);" +
    		"}";

    private static final String vShaderCode =
    		"attribute vec4 position;" +
    		"attribute vec2 uv;" +
    		"varying vec2 texCoord;" +
    		"void main() {" +
    		"  gl_Position = position;" +
    		"  texCoord = uv;" +
    		"}";

    private static final String fShaderCode =
    		"precision mediump float;" +
    		"varying vec2 texCoord;" +
    		"uniform sampler2D texOnQuad;" +
    		"void main() {" +
    		"  gl_FragColor = texture2D(texOnQuad, texCoord) + vec4(0.5f, 0.2f, 0.0f, 1.0f);" +
    		"}";

    private static final float[] quadVertices = {
    		-1.0f,  1.0f, 0.0f,   // top left
    		-1.0f, -1.0f, 0.0f,   // bottom left
    		 1.0f, -1.0f, 0.0f,   // bottom right
    		 1.0f,  1.0f, 0.0f    // top right
    };

    private static final float[] quadUV = {
	    	0.0f, 1.0f, // top left
	    	0.0f, 0.0f, // bottom left
	    	1.0f, 0.0f, // bottom right
	    	1.0f, 1.0f  // top right
    };

    private static short[] drawList = {
    		0, 1, 2, 0, 2, 3
    };

    private RenderTexture cameraRenderTexture = null;
    private final RenderTexture renderTexture = new RenderTexture();
    private int width = 0;
    private int height = 0;
	private FloatBuffer vertexBuffer;
	private final int vertexStride = 3 * Float.SIZE / 8;
	private FloatBuffer uvBuffer;
	private final int uvStride = 2 * Float.SIZE / 8;
	private ShortBuffer drawListBuffer;
	private final Shader shader = new Shader();
	private final Shader shaderBasic = new Shader();

	public void setCameraRenderTexture(RenderTexture cameraRenderTexture) {
		this.cameraRenderTexture = cameraRenderTexture;
		setupFBO();
	}

	public RenderTexture getSharedRenderTexture() {
		return this.renderTexture;
	}

	public void init() {
    	ByteBuffer bb = ByteBuffer.allocateDirect(quadVertices.length * Float.SIZE / 8);
    	bb.order(ByteOrder.nativeOrder());
    	this.vertexBuffer = bb.asFloatBuffer();
    	this.vertexBuffer.put(quadVertices);
    	this.vertexBuffer.position(0);

    	bb = ByteBuffer.allocateDirect(quadUV.length * Float.SIZE / 8);
    	bb.order(ByteOrder.nativeOrder());
    	this.uvBuffer = bb.asFloatBuffer();
    	this.uvBuffer.put(quadUV);
    	this.uvBuffer.position(0);

    	bb = ByteBuffer.allocateDirect(drawList.length * Short.SIZE / 8);
    	bb.order(ByteOrder.nativeOrder());
    	this.drawListBuffer = bb.asShortBuffer();
    	this.drawListBuffer.put(drawList);
    	this.drawListBuffer.position(0);

		final PassShader passShaderBasic = this.shaderBasic.addPassShader(0);
		passShaderBasic.createProgram();
		passShaderBasic.attachVertexShader(vShaderBasicCode);
		passShaderBasic.attachFragmentShader(fShaderBasicCode);
		passShaderBasic.linkProgram();

    	this.shaderBasic.setShaderBuffer(
    			"position", 3, this.vertexStride, this.vertexBuffer);

    	this.shaderBasic.setShaderBuffer(
    			"uv", 2, this.uvStride, this.uvBuffer);

		this.shaderBasic.setPass(0);


		final PassShader passShader = this.shader.addPassShader(0);
		passShader.createProgram();
		passShader.attachVertexShader(vShaderCode);
		passShader.attachFragmentShader(fShaderCode);
		passShader.linkProgram();

    	this.shader.setShaderBuffer(
    			"position", 3, this.vertexStride, this.vertexBuffer);

    	this.shader.setShaderBuffer(
    			"uv", 2, this.uvStride, this.uvBuffer);

		this.shader.setPass(0);
	}

	public void setDim(int width, int height) {
    	this.width = width;
    	this.height = height;

    	this.renderTexture.init(width, height);
    	this.renderTexture.create();

    	//setCameraRenderTexture(renderTexture);
	}

	private int fbHandle = -1;
	private int depthFbHandle = -1;

	public void setupFBO() {
		final int[] names = { -1 };

		// FB
		GLES20.glGenFramebuffers(1, names, 0);
		this.fbHandle = names[0];

		// depth
		GLES20.glGenRenderbuffers(1, names, 0);
		this.depthFbHandle = names[0];

		GLES20.glBindRenderbuffer(GLES20.GL_RENDERBUFFER, this.depthFbHandle);
		GLES20.glRenderbufferStorage(
				GLES20.GL_RENDERBUFFER, GLES20.GL_DEPTH_COMPONENT16,
				this.width, this.height);
		GLES20.glBindRenderbuffer(GLES20.GL_RENDERBUFFER, 0);
	}

	private boolean glParamsInitialized = false;

	private void initGLParams() {
		GLES20.glEnable(GLES20.GL_DEPTH_TEST);
		GLES20.glClearDepthf(1.0f);
		GLES20.glDepthFunc(GLES20.GL_LEQUAL);  // Passes if the incoming depth value is less than or equal to the stored depth value.
		GLES20.glDepthMask(true); // enable writing into the depth buffer

		// cull backface
		GLES20.glEnable(GLES20.GL_CULL_FACE);
		GLES20.glCullFace(GLES20.GL_BACK);

		GLES20.glEnable(GLES20.GL_TEXTURE_2D);

		this.glParamsInitialized = true;
	}

	public void preRenderToTexture() {
		Texture.resetTextureCounter();

		GLES20.glBindFramebuffer(GLES20.GL_FRAMEBUFFER, this.fbHandle);

		GLES20.glFramebufferTexture2D(
				GLES20.GL_FRAMEBUFFER, GLES20.GL_COLOR_ATTACHMENT0,
                GLES20.GL_TEXTURE_2D, this.cameraRenderTexture.getTextureHandle(), 0);

		GLES20.glFramebufferRenderbuffer(
				GLES20.GL_FRAMEBUFFER, GLES20.GL_DEPTH_ATTACHMENT,
				GLES20.GL_RENDERBUFFER, this.depthFbHandle);

		final int fbStatus = GLES20.glCheckFramebufferStatus(GLES20.GL_FRAMEBUFFER);
		if (fbStatus != GLES20.GL_FRAMEBUFFER_COMPLETE) {
			throw new FrameBufferIncompleteException(
					"Could not bind frame buffer: " + fbStatus);
		}

		GLES20.glViewport(0, 0, this.width, this.height);

//		if (!this.glParamsInitialized) {
//			initGLParams();
//		}

    	GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT);
    	GLES20.glClearColor(0.5f, 0.5f, 0.5f, 1.0f);
	}

	public void renderToTexture() {
		preRenderToTexture();

    	this.shaderBasic.use();

    	GLES20.glDrawElements(GLES20.GL_TRIANGLES, drawList.length,
                GLES20.GL_UNSIGNED_SHORT, this.drawListBuffer);

    	this.shaderBasic.releaseAllShaderBuffers();

    	Utils.checkGlError("hihi !");

    	this.cameraRenderTexture.swapPrevAndCurrentTexture();
	}

	public void renderToQuad() {
		Texture.resetTextureCounter();

    	GLES20.glBindFramebuffer(GLES20.GL_FRAMEBUFFER, 0);

    	GLES20.glViewport(0, 0, this.width, this.height);

    	GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT);
    	GLES20.glClearColor(0.5f, 0.5f, 0.5f, 1.0f);

    	this.shader.setShaderTexture("texOnQuad", this.renderTexture.getPrevTextureHandle());

    	this.shader.use();

    	GLES20.glDrawElements(GLES20.GL_TRIANGLES, drawList.length,
                GLES20.GL_UNSIGNED_SHORT, this.drawListBuffer);

    	this.shader.releaseAllShaderBuffers();
	}
}
